@@ -908,6 +908,12 @@ AccessibleTable_getColumnExtentAt (AccessibleTable *obj,
                                    long int row,
                                    long int column);
 
+SPIBoolean
+AccessibleTable_getRowColumnExtentsAtIndex (AccessibleTable *obj,
+					    long int index, long int *row, long int *col, 
+					    long int *row_extents, long int *col_extents, 
+					    long int *is_selected);
+
 Accessible *
 AccessibleTable_getRowHeader (AccessibleTable *obj,
 			      long int row);
@@ -1077,6 +1083,14 @@ AccessibleText_setSelection (AccessibleText *obj,
 			     long int selectionNum,
 			     long int startOffset,
 			     long int endOffset);
+AccessibleAttributeSet *
+AccessibleText_getAttributeRun  (AccessibleText *obj,
+				 long int offset,
+				 long int *startOffset,
+				 long int *endOffset,
+				 long int includeDefaults);
+AccessibleAttributeSet *
+AccessibleText_getDefaultAttributeSet (AccessibleText *obj);
 
 /* AccessibleValue Function Prototypes:  */
 
